<?php

namespace Drupal\pdftemplate\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\pdftemplate\PDFTemplateInterface;

/**
 * Defines the pdftemplate entity class.
 *
 * @ContentEntityType(
 *   id = "pdftemplate",
 *   label = @Translation("PDFTemplate"),
 *   label_collection = @Translation("PDFTemplates"),
 *   handlers = {
 *     "view_builder" = "Drupal\pdftemplate\PdftemplateViewBuilder",
 *     "list_builder" = "Drupal\pdftemplate\PdftemplateListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "add" = "Drupal\pdftemplate\Form\PdftemplateForm",
 *       "edit" = "Drupal\pdftemplate\Form\PdftemplateForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "pdftemplate",
 *   admin_permission = "access pdftemplate overview",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "add-form" = "/admin/config/services/pdftemplate/add",
 *     "canonical" = "/admin/config/services/pdftemplate/{pdftemplate}",
 *     "edit-form" = "/admin/config/services/pdftemplate/{pdftemplate}/edit",
 *     "delete-form" = "/admin/config/services/pdftemplate/{pdftemplate}/delete",
 *     "collection" = "/admin/config/services/pdftemplate"
 *   },
 * )
 */
class PDFTemplate extends ContentEntityBase implements PDFTemplateInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getVariables() {
    $variables = $this->get('variables')->getValue();
    return $variables[0];
  }

  /**
   * {@inheritdoc}
   */
  public function setVariables($variables) {
    $this->set('variables', $variables);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function addVariable($var) {
    $this->set('variables', $var);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The title of the pdftemplate entity.'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['template'] = BaseFieldDefinition::create('file')
      ->setLabel('Template')
      ->setSettings([
        'uri_scheme' => 'private',
        'file_directory' => 'pdftemplate',
        'file_extensions' => 'odt',
      ])
      ->setDisplayOptions('view', array(
        'label' => 'above',
        'type' => 'file',
        'weight' => -3,
      ))
      ->setDisplayOptions('form', array(
        'type' => 'file',
        'weight' => -1,
      ))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['variables'] = BaseFieldDefinition::create('map');

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the pdftemplate was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the pdftemplate was last edited.'));

    return $fields;
  }

}
