<?php

namespace Drupal\pdftemplate\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a PDFTemplate form.
 */
class VariablesForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pdftemplate_variables';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $template_id = \Drupal::routeMatch()->getParameter('pdftemplate');
    $template = \Drupal::entityTypeManager()->getStorage('pdftemplate')->load($template_id);
    $variables = $template->getVariables();
    $form['template_id'] = [
      '#type' => 'value',
      '#value' => $template_id
    ];
    $form['variables'] = [
      '#type' => 'table',
      '#title' => 'Tenplate variables',
      '#header' => ['Variable name', 'Type', 'Delete'],
    ];
    $i = 0;
    foreach($variables as $key => $var) {
      $form['variables'][$i]['name'] = [
        '#type' => 'textfield',
        '#size' => 25,
        '#title' => $this->t('Variable Name'),
        '#field_prefix' => '{',
        '#field_suffix' => '}',
        '#default_value' => $var['name']
      ];
      $form['variables'][$i]['type'] = [
        '#type' => 'select',
        '#title' => $this->t('Type'),
        '#options' => $this->variableTypeOptions(),
        '#default_value' => $var['type']
      ];
      $form['variables'][$i]['delete'] = [
        '#type' => 'checkbox',
      ];
      $i++;
    }
    $form['variables_add'] = [
      '#type' => 'table',
      '#title' => 'Add Template variable',
      '#header' => ['Add Variable name', 'Type'],
    ];
    $form['variables_add']['add']['name'] = [
      '#type' => 'textfield',
      '#size' => 25,
      '#title' => $this->t('Variable Name'),
      '#field_prefix' => '{',
      '#field_suffix' => '}'
    ];
    $form['variables_add']['add']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type'),
      '#options' => $this->variableTypeOptions()
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];
    return $form;
  }

  public function variableTypeOptions() {
    return [
      'string' => $this->t('String'),
      'image' => $this->t('Image')
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {}

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $template_id = $form_state->getValue('template_id');
    $template = \Drupal::entityTypeManager()->getStorage('pdftemplate')->load($template_id);

    $variables = [];
    foreach($form_state->getValue('variables') as $var) {
      if(!$var['delete']) {
        $variables[$var['name']] = [
          'name' => $var['name'],
          'type' => $var['type']
        ];
      }
    }

    $variables_add = $form_state->getValue('variables_add');
    if(isset($variables_add['add']['name']) && $variables_add['add']['name'] != '') {
      $variables[$variables_add['add']['name']] = [
        'name' => $variables_add['add']['name'],
        'type' => $variables_add['add']['type']
      ];
    }

    $template->setVariables($variables);
    $template->save();

    $this->messenger()->addStatus($this->t('The message has been sent.'));
  }

}
